#!/usr/bin/env python3
import os
import tomllib

_file_version = os.path.relpath(__file__)
_dir_version = os.path.abspath(os.path.dirname(_file_version))
base_dir = os.path.abspath(os.path.join(_dir_version, '..'))
file_py_project = os.path.join(base_dir, 'pyproject.toml')
__version__ = ""

try:
    with open(file_py_project, 'rb') as f:
        _data = tomllib.load(f)
except Exception as e:
    print(e)
else:
    __version__ = _data['project']['version']






#!/usr/bin/env python3
#
"""
    Esté módulo serve para manipulação de arquivos, diretórios e documentos
entre outros. Não depende de módulos externos apenas de builtins e stdlib.
"""
from __future__ import annotations
from enum import Enum
import os
import json
import platform
from pathlib import Path
from hashlib import md5

# Windows / Linux / ...
KERNEL_TYPE = platform.system()


class ExtensionFiles(Enum):
    PNG = '.png'
    JPG = '.jpg'
    JPEG = '.jpeg'
    SVG = '.svg'
    PDF = '.pdf'
    XLSX = '.xlsx'
    XLS = '.xls'
    CSV = '.csv'
    ODS = '.ods'
    JSON = '.json'


class LibraryDocs(Enum):
    """
        Enum para tipos de documentos como imagens, PDFs, Planilhas e JSON.
    """

    IMAGE = ['.png', '.jpg', '.jpeg', '.svg']
    PDF = ['.pdf']
    DOCUMENTS = ['.png', '.jpg', '.jpeg', '.svg', '.pdf']
    #
    EXCEL = ['.xlsx', '.xls']
    CSV = ['.csv', '.txt']
    ODS = ['.ods']
    #
    SHEET = ['.csv', '.txt', '.xlsx', '.xls', '.ods']
    JSON = ['.json']
    #
    ALL_DOCUMENTS = [
        '.png', '.jpg', '.jpeg', '.svg',
        '.csv', '.txt', '.xlsx', '.xls', '.ods',
        '.pdf',
        '.json',
    ]
    #
    ALL = None


class File(object):
    def __init__(self, filename: str):
        if os.path.isdir(filename):
            raise ValueError(f'{__class__.__name__} File() não pode ser um diretório.')
        self.filename: str = os.path.abspath(filename)
        self.__path: Path = Path(self.filename)

    @property
    def path(self) -> Path:
        return self.__path

    @path.setter
    def path(self, new: Path):
        if not isinstance(new, Path):
            return
        self.__path = new

    def __eq__(self, value):
        if not isinstance(value, File):
            return NotImplemented
        return self.absolute() == value.absolute()

    def __hash__(self):
        return self.absolute().__hash__()

    def is_image(self) -> bool:
        try:
            return True if self.extension() in LibraryDocs.IMAGE.value else False
        except:
            return False

    def is_pdf(self) -> bool:
        try:
            return True if self.extension() in LibraryDocs.PDF.value else False
        except:
            return False

    def is_excel(self) -> bool:
        try:
            return True if self.extension() in LibraryDocs.EXCEL.value else False
        except:
            return False

    def is_csv(self) -> bool:
        try:
            return True if self.extension() in LibraryDocs.CSV.value else False
        except:
            return False

    def is_sheet(self) -> bool:
        try:
            return True if self.extension() in LibraryDocs.SHEET.value else False
        except:
            return False

    def is_json(self) -> bool:
        try:
            return True if self.extension() in LibraryDocs.JSON.value else False
        except:
            return False

    def is_ods(self):
        try:
            return True if self.extension() in LibraryDocs.ODS.value else False
        except:
            return False

    def update_extension(self, e: str) -> File:
        """
            Retorna uma instância de File() no mesmo diretório com a nova
        extensão informada.
        """
        current = self.extension()
        full_path = self.absolute().replace(current, '')
        return File(os.path.join(f'{full_path}{e}'))

    def get_text(self) -> str | None:
        try:
            return self.__path.read_text()
        except Exception as e:
            print(e)
            return None

    def write_string(self, s: str):
        if s is None:
            return False
        try:
            self.__path.write_text(s)
        except Exception as e:
            print(e)
            return False
        else:
            return True

    def write_list(self, items: list[str]) -> bool:
        # Abrindo o arquivo em modo de escrita
        if len(items) == 0:
            return False
        try:
            with open(self.filename, "w", encoding="utf-8") as file:
                for string in items:
                    file.write(string + "\n")  # Adiciona uma quebra de linha após cada string
        except Exception as e:
            print(e)
            return False
        else:
            return True

    def name(self):
        e = self.extension()
        if (e is None) or (e == ''):
            return os.path.basename(self.filename)
        return os.path.basename(self.filename).replace(e, '')

    def name_absolute(self) -> str:
        e = self.extension()
        if (e is None) or (e == ''):
            return self.filename
        return self.filename.replace(e, '')

    def extension(self) -> str:
        return self.__path.suffix

    def dirname(self) -> str:
        return os.path.dirname(self.filename)

    def basename(self) -> str:
        return os.path.basename(self.filename)

    def exists(self) -> bool:
        return self.__path.exists()

    def absolute(self) -> str:
        return self.filename

    def size(self):
        return os.path.getsize(self.filename)

    def md5(self) -> str | None:
        """Retorna a hash md5 de um arquivo se ele existir no disco."""
        if not self.path.exists():
            return None
        _hash_md5 = md5()
        with open(self.absolute(), "rb") as f:
            for _block in iter(lambda: f.read(4096), b""):
                _hash_md5.update(_block)
        return _hash_md5.hexdigest()


class Directory(object):
    def __init__(self, dirpath: str):
        self.dirpath: str = os.path.abspath(dirpath)
        self.path: Path = Path(self.dirpath)

    def __eq__(self, value):
        if not isinstance(value, Directory):
            return NotImplemented
        return self.absolute() == value.absolute()

    def __hash__(self):
        return self.absolute().__hash__()

    def iterpaths(self) -> list[Path]:
        return list(self.path.rglob('*'))

    def __content_recursive(self) -> list[File]:
        _paths = self.iterpaths()
        values = []
        for p in _paths:
            if p.is_file():
                values.append(
                    File(os.path.abspath(p.absolute()))
                )
        return values

    def __content_no_recursive(self) -> list[File]:
        content_files: list[str] = os.listdir(self.absolute())
        values: list[File] = []
        for file in content_files:
            fp: str = os.path.join(self.absolute(), file)
            if os.path.isfile(fp):
                values.append(
                    File(os.path.abspath(fp))
                )
        return values

    def content_files(self, *, recursive: bool = True) -> list[File]:
        if recursive:
            return self.__content_recursive()
        return self.__content_no_recursive()

    def content_dirs(self, recursive: bool = True) -> list[Directory]:
        values: list[Directory] = []
        if recursive:
            _paths = self.iterpaths()
            for p in _paths:
                if p.is_dir():
                    values.append(
                        Directory(os.path.abspath(p.absolute()))
                    )
        else:
            _paths = os.listdir(self.absolute())
            for d in _paths:
                _dirpath = os.path.join(self.absolute(), d)
                if os.path.isdir(_dirpath):
                    values.append(
                        Directory(os.path.abspath(_dirpath))
                    )
        return values

    def basename(self) -> str:
        return os.path.basename(self.absolute())

    def mkdir(self):
        try:
            os.makedirs(self.absolute())
        except:
            pass

    def absolute(self) -> str:
        return self.dirpath

    def concat(self, d: str, create: bool = False) -> Directory:
        if create:
            if not os.path.exists(os.path.join(self.absolute(), d)):
                try:
                    os.makedirs(os.path.join(self.absolute(), d))
                except Exception as err:
                    print(err)
        return Directory(os.path.join(self.absolute(), d))

    def parent(self) -> Directory:
        return Directory(
            os.path.abspath(self.path.parent)
        )

    def join_file(self, name: str) -> File:
        return File(
            os.path.join(self.absolute(), name)
        )


class InputFiles(object):
    """
        Obter uma lista de arquivos/documentos do diretório informado.
    """

    def __init__(self, d: Directory, *, maxFiles: int = 5000):
        if not isinstance(d, Directory):
            raise ValueError(f'{__class__.__name__}\nUse: Directory(), não {type(d)}')
        self.input_dir: Directory = d
        self.maxFiles: int = maxFiles

    @property
    def images(self) -> list[File]:
        return self.get_files(file_type=LibraryDocs.IMAGE)

    @property
    def pdfs(self) -> list[File]:
        return self.get_files(file_type=LibraryDocs.PDF)

    @property
    def sheets(self) -> list[File]:
        return self.get_files(file_type=LibraryDocs.SHEET)

    def get_files_with(self, *, infile: str, sort: bool = True) -> list[File]:
        """
            Retorna arquivos que contém a ocorrência (infile) no nome absoluto.
        """
        content_files: list[File] = []
        count: int = 0
        paths = self.input_dir.iterpaths()
        for file in paths:
            if not file.is_file():
                continue
            if infile in os.path.abspath(file.absolute()):
                content_files.append(
                    File(os.path.abspath(file.absolute()))
                )
                count += 1
            if count >= self.maxFiles:
                break
        return content_files

    def __get_files_recursive(self, *, file_type: LibraryDocs, sort: bool) -> list[File]:
        #
        _paths: list[Path] = self.input_dir.iterpaths()
        _all_files = []
        count: int = 0
        if file_type == LibraryDocs.ALL:
            # Todos os tipos de arquivos
            for p in _paths:
                if not p.is_file():
                    continue
                _all_files.append(
                    File(os.path.abspath(p.absolute()))
                )
                count += 1
                if count >= self.maxFiles:
                    break
        else:
            # Arquivos especificados em LibraryDocs
            for p in _paths:
                if not p.is_file():
                    continue
                if (p.suffix is None) or (p.suffix == ''):
                    continue
                if p.suffix in file_type.value:
                    _all_files.append(
                        File(os.path.abspath(p.absolute()))
                    )
                    count += 1
                if count >= self.maxFiles:
                    break
        if sort:
            _all_files.sort(key=File.absolute)
        return _all_files

    def __get_files_no_recursive(self, *, file_type: LibraryDocs, sort: bool) -> list[File]:
        _content_files: list[File] = self.input_dir.content_files(recursive=False)
        _all_files: list[File] = []
        count: int = 0

        if file_type == LibraryDocs.ALL:
            # Todos os tipos de arquivos
            for file in _content_files:
                _all_files.append(file)
                count += 1
                if count == self.maxFiles:
                    break
        else:
            # Arquivos especificados em LibraryDocs
            for file in _content_files:
                if file.extension() in file_type.value:
                    _all_files.append(file)
                    count += 1
                    if count == self.maxFiles:
                        break
        if sort:
            _all_files.sort(key=File.absolute)
        return _all_files

    def get_files(
            self, *,
            file_type: LibraryDocs = LibraryDocs.ALL_DOCUMENTS,
            sort: bool = True,
            recursive: bool = True
    ) -> list[File]:
        """
            Retorna uma lista de File() conforme o tipo de arquivo
        especificado.
        - LibraryDocs.ALL_DOCUMENTS => Retorna todos os documentos do diretório.
        - LibraryDocs.EXCEL         => Retorna arquivos que são planilhas excel.
        - ...
        
        """
        if recursive:
            return self.__get_files_recursive(file_type=file_type, sort=sort)
        return self.__get_files_no_recursive(file_type=file_type, sort=sort)


class JsonData(object):
    """
        Representação de um dado JSON apartir de uma string python.
    """

    def __init__(self, string: str):
        if not isinstance(string, str):
            raise ValueError(f'{__class__.__name__} o JSON informado precisa ser do tipo string, não {type(string)}')
        self.jsonString: str = string

    def is_null(self) -> bool:
        if (self.jsonString is None) or (self.jsonString == ''):
            return True
        return False

    def to_string(self) -> str:
        return self.jsonString

    def to_dict(self) -> dict[str, object]:
        """
            Exportar/Converter o dado atual em um dicionário python.
        """
        return json.loads(self.jsonString)

    def to_file(self, f: File):
        """Exportar o dado atual para um arquivo .json"""
        _data: str = json.loads(self.jsonString)
        with open(f.absolute(), "w", encoding="utf-8") as file:
            json.dump(_data, file, indent=4, ensure_ascii=False)


class JsonConvert(object):
    """
        Conversão de um dado JSON em dados python
    """

    def __init__(self, jsonData: JsonData):
        self.jsonData: JsonData = jsonData

    def to_json_data(self) -> JsonData:
        return self.jsonData

    @classmethod
    def from_file(cls, file: File) -> JsonConvert:
        """
            Gerar um dado JsonData apartir de arquivo .json
        """
        # Ler o arquivo e carregar o JSON em um dicionário Python
        data = None
        try:
            with open(file.absolute(), "r", encoding="utf-8") as fp:
                data: str = json.load(fp)
        except Exception as e:
            print(f'{__class__.__name__}\n{e}')
            return cls(JsonData(''))
        else:
            return cls(JsonData(json.dumps(data, indent=4, ensure_ascii=False, sort_keys=True)))

    @classmethod
    def from_string_json(cls, data: str) -> JsonConvert:
        """
            Gerar um dado JsonData apartir de uma string.
        """
        json_string = json.dumps(data, indent=4, ensure_ascii=False, sort_keys=True)
        return cls(JsonData(json_string))

    @classmethod
    def from_dict(cls, data: dict[str, object]) -> JsonConvert:
        """
            Converte um dicionário em objeto JSON/JsonData.
        """
        if not isinstance(data, dict):
            raise ValueError(f'{__class__.__name__} Informe um JSON em formato dict, não {type(data)}')
        json_string = json.dumps(data, indent=4, ensure_ascii=False, sort_keys=True)
        return cls(JsonData(json_string))


class UserFileSystem(object):
    """
        Diretórios comuns para cache e configurações de usuário.
    """

    def __init__(self, base_home: Directory = Directory(os.path.abspath(Path().home()))):
        self.baseHome: Directory = base_home
        self.userDownloads: Directory = self.baseHome.concat('Downloads', create=True)
        self.userVarDir: Directory = self.baseHome.concat('var', create=True)

    def config_dir(self) -> Directory:
        return self.userVarDir.concat('config', create=True)

    def cache_dir(self) -> Directory:
        return self.userVarDir.concat('cache', create=True)


class UserAppDir(object):
    """
        Diretório comun para cache e configurações do aplicativo.
    """

    def __init__(self, appname: str, *, user_file_system: UserFileSystem = UserFileSystem()):
        self.appname = appname
        self.userFileSystem: UserFileSystem = user_file_system
        self.workspaceDirApp: Directory = self.userFileSystem.userDownloads.concat(self.appname, create=True)
        self.installDir: Directory = self.userFileSystem.userVarDir.concat('opt').concat(self.appname, create=True)

    def cache_dir_app(self) -> Directory:
        return self.userFileSystem.cache_dir().concat(self.appname, create=True)

    def config_dir_app(self) -> Directory:
        return self.userFileSystem.config_dir().concat(self.appname, create=True)



#!/usr/bin/env python3
#
from __future__ import annotations
from typing import List, Dict
from pip._internal.cli.main import main as pip_main


PACKAGES: Dict[str, str] = {
    'pandas': 'https://github.com/pandas-dev/pandas/archive/refs/heads/main.zip',
    'fitz': 'https://github.com/pymupdf/PyMuPDF/archive/refs/heads/main.zip',
    'pytesseract': 'https://github.com/madmaze/pytesseract/archive/refs/heads/master.zip',
}

STATUS: Dict[str, bool] = {}


def install_packages(ignore_items: List[str] = []):

    for name, url in PACKAGES.items():
        if name in ignore_items:
            continue
        print('---------------------------------')
        print(name, url)
        try:
            pip_main(['install', url])
        except Exception as e:
            print(e)
            print()


def main():
    install_packages()


if __name__ == '__main__':
    main()





#!/usr/bin/env python3
from soup_files import __version__


def test():
    from soup_files import (
        File, Directory, UserFileSystem, LibraryDocs, InputFiles
    )
    pass


def main():
    print(f' soup_files - versão: {__version__}')
    test()


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
from typing import Optional
from abc import ABC, abstractmethod


class ABCProgressBar(ABC):
    """
        Barra de progresso Abstrata
    """

    def __init__(self):
        super().__init__()
        self._num_progress: float = 0
        self.pbar_real: object = None

    @property
    def num_progress(self) -> float:
        return self._num_progress

    @num_progress.setter
    def num_progress(self, new: float):
        if isinstance(new, float):
            self._num_progress = new
            return
        try:
            _prog = float(new)
        except Exception as e:
            print(e)
        else:
            self._num_progress = _prog

    @abstractmethod
    def set_percent(self, percent: float):
        """Seta o progresso com float de porcentagem, ex: '42.8'"""
        pass

    @abstractmethod
    def set_text(self, text: str):
        """Seta um texto indicando a situação atual"""
        pass

    def start(self):
        """Inicia a barra de progresso (pode ser vazio dependendo da implementação)"""
        pass

    def stop(self):
        """Para a barra de progresso (pode ser vazio dependendo da implementação)"""
        pass


class ProgressBarSimple(ABCProgressBar):
    """Barra de progresso simples para mostrar no terminal."""

    def __init__(self, simple_pbar=None):
        super().__init__()
        self.pbar_real = simple_pbar
        self._text: str = 'Aguarde!'
        self.num_progress: float = 0

    def set_percent(self, percent: float):
        if not isinstance(percent, float):
            return
        if len(f'{percent}') > 4:
            percent = round(float(percent), 2)
        self.num_progress = percent
        #print(f'[{self.num_progress}%] {self._text}', end='\r')

    def set_text(self, text: str):
        self._text = text
        print(f'[{self.num_progress}%] {self._text}', end='\r')

    def start(self):
        pass

    def stop(self):
        pass


class ProgressBarAdapter(object):
    def __init__(self, progress_bar: ABCProgressBar = ProgressBarSimple()):
        self.pbar_implement: ABCProgressBar = progress_bar

    def get_current_percent(self) -> float:
        return self.pbar_implement.num_progress

    def update_text(self, text: str = "-"):
        self.pbar_implement.set_text(text)

    def update_percent(self, percent: float = 0):
        if not isinstance(percent, float):
            try:
                percent = float(percent)
            except Exception as e:
                print(f'{__class__.__name__} {e}')
                percent = 0
        self.pbar_implement.set_percent(percent)

    def update(self, percent: float, status: str = "-"):
        self.update_percent(percent)
        self.update_text(status)

    def start(self):
        self.pbar_implement.start()

    def stop(self):
        self.pbar_implement.stop()


class TextProgress(object):

    def __init__(self, start: int = 0, total: int = 1, *, pbar: ProgressBarAdapter = None):
        if total == 0:
            raise ValueError(f'{__class__.__name__} o total não pode ser 0')
        if pbar is None:
            self.__pbar: ProgressBarAdapter = ProgressBarAdapter()
        else:
            self.__pbar: ProgressBarAdapter = pbar
        self.__start: int = start
        self.__total: int = total
        self.__text: str = 'Aguarde!'

    @property
    def total(self) -> int:
        return self.__total

    @total.setter
    def total(self, new: int):
        if new == 0:
            return
        self.__total = new

    @property
    def start(self) -> int:
        return self.__start

    @start.setter
    def start(self, new: int):
        self.__start = new

    @property
    def pbar(self) -> ProgressBarAdapter:
        return self.__pbar

    @pbar.setter
    def pbar(self, new: ProgressBarAdapter):
        self.__pbar = new

    @property
    def text(self) -> str:
        return self.__text

    @text.setter
    def text(self, new: str):
        self.__text = new

    def start_pbar(self):
        self.__pbar.start()

    def stop_pbar(self):
        self.__pbar.stop()

    def set_update(self):
        self.pbar.update(
            (self.__start / self.__total) * 100,
            f'[{self.__start}/{self.__total}] {self.__text}]'
        )
        self.__start += 1


class CreatePbar(object):
    _instance: Optional['CreatePbar'] = None

    def __new__(cls, pbar: 'ProgressBarAdapter' = None):
        if cls._instance is None:
            cls._instance = super(CreatePbar, cls).__new__(cls)
            cls._instance._initialized = False
        return cls._instance

    def __init__(self, pbar: 'ProgressBarAdapter' = None):
        if not self._initialized:
            self.pbar: ProgressBarAdapter = pbar or ProgressBarAdapter()
            self.text_progress: TextProgress = TextProgress(0, 1, pbar=self.pbar)
            self._initialized = True

    def get(self) -> 'ProgressBarAdapter':
        return self.pbar

    def get_text_progress(self) -> TextProgress:
        return self.text_progress




#!/usr/bin/env python3

from .version import __version__
from .progress import (
    ProgressBarAdapter, ProgressBarSimple, ABCProgressBar,
    CreatePbar, TextProgress
)
from .files import (
    File,
    Directory,
    JsonData,
    JsonConvert,
    InputFiles,
    LibraryDocs,
    UserFileSystem,
    UserAppDir,
    KERNEL_TYPE,
)

